---
title: sql_select
type: processor
status: stable
categories: ["Integration"]
---

<!--
     THIS FILE IS AUTOGENERATED!

     To make changes please edit the contents of:
     lib/processor/sql_select.go
-->

import Tabs from '@theme/Tabs';
import TabItem from '@theme/TabItem';

Runs an SQL select query against a database and returns the result as an array of objects, one for each row returned, containing a key for each column queried and its value.

Introduced in version 3.59.0.


<Tabs defaultValue="common" values={[
  { label: 'Common', value: 'common', },
  { label: 'Advanced', value: 'advanced', },
]}>

<TabItem value="common">

```yml
# Common config fields, showing default values
label: ""
sql_select:
  driver: ""
  dsn: ""
  table: ""
  columns: []
  where: ""
  args_mapping: ""
```

</TabItem>
<TabItem value="advanced">

```yml
# All config fields, showing default values
label: ""
sql_select:
  driver: ""
  dsn: ""
  table: ""
  columns: []
  where: ""
  args_mapping: ""
  prefix: ""
  suffix: ""
  conn_max_idle_time: ""
  conn_max_life_time: ""
  conn_max_idle: 0
  conn_max_open: 0
```

</TabItem>
</Tabs>

If the query fails to execute then the message will remain unchanged and the error can be caught using error handling methods outlined [here](/docs/configuration/error_handling).

## Examples

<Tabs defaultValue="Table Query (PostgreSQL)" values={[
{ label: 'Table Query (PostgreSQL)', value: 'Table Query (PostgreSQL)', },
]}>

<TabItem value="Table Query (PostgreSQL)">


Here we query a database for columns of footable that share a `user_id`
with the message `user.id`. A [`branch` processor](/docs/components/processors/branch)
is used in order to insert the resulting array into the original message at the
path `foo_rows`:

```yaml
pipeline:
  processors:
    - branch:
        processors:
          - sql_select:
              driver: postgres
              dsn: postgres://foouser:foopass@localhost:5432/testdb?sslmode=disable
              table: footable
              columns: [ '*' ]
              where: user_id = ?
              args_mapping: '[ this.user.id ]'
        result_map: 'root.foo_rows = this'
```

</TabItem>
</Tabs>

## Fields

### `driver`

A database [driver](#drivers) to use.


Type: `string`  
Options: `mysql`, `postgres`, `clickhouse`, `mssql`.

### `dsn`

A Data Source Name to identify the target database.

#### Drivers

The following is a list of supported drivers, their placeholder style, and their respective DSN formats:

| Driver | Data Source Name Format |
|---|---|
| `clickhouse` | [`clickhouse://[username[:password]@][netloc][:port]/dbname[?param1=value1&...&paramN=valueN]`](https://github.com/ClickHouse/clickhouse-go#dsn) |
| `mysql` | `[username[:password]@][protocol[(address)]]/dbname[?param1=value1&...&paramN=valueN]` |
| `postgres` | `postgres://[user[:password]@][netloc][:port][/dbname][?param1=value1&...]` |
| `mssql` | `sqlserver://[user[:password]@][netloc][:port][?database=dbname&param1=value1&...]` |

Please note that the `postgres` driver enforces SSL by default, you can override this with the parameter `sslmode=disable` if required.


Type: `string`  

```yml
# Examples

dsn: clickhouse://username:password@host1:9000,host2:9000/database?dial_timeout=200ms&max_execution_time=60

dsn: foouser:foopassword@tcp(localhost:3306)/foodb

dsn: postgres://foouser:foopass@localhost:5432/foodb?sslmode=disable
```

### `table`

The table to query.


Type: `string`  

```yml
# Examples

table: foo
```

### `columns`

A list of columns to query.


Type: `array`  

```yml
# Examples

columns:
  - '*'

columns:
  - foo
  - bar
  - baz
```

### `where`

An optional where clause to add. Placeholder arguments are populated with the `args_mapping` field. Placeholders should always be question marks, and will automatically be converted to dollar syntax when the postgres or clickhouse drivers are used.


Type: `string`  

```yml
# Examples

where: meow = ? and woof = ?

where: user_id = ?
```

### `args_mapping`

An optional [Bloblang mapping](/docs/guides/bloblang/about) which should evaluate to an array of values matching in size to the number of placeholder arguments in the field `where`.


Type: `string`  

```yml
# Examples

args_mapping: root = [ this.cat.meow, this.doc.woofs[0] ]

args_mapping: root = [ meta("user.id") ]
```

### `prefix`

An optional prefix to prepend to the query (before SELECT).


Type: `string`  

### `suffix`

An optional suffix to append to the select query.


Type: `string`  

### `conn_max_idle_time`

An optional maximum amount of time a connection may be idle. Expired connections may be closed lazily before reuse. If value <= 0, connections are not closed due to a connection's idle time.


Type: `string`  

### `conn_max_life_time`

An optional maximum amount of time a connection may be reused. Expired connections may be closed lazily before reuse. If value <= 0, connections are not closed due to a connection's age.


Type: `string`  

### `conn_max_idle`

An optional maximum number of connections in the idle connection pool. If conn_max_open is greater than 0 but less than the new conn_max_idle, then the new conn_max_idle will be reduced to match the conn_max_open limit. If value <= 0, no idle connections are retained. The default max idle connections is currently 2. This may change in a future release.


Type: `int`  

### `conn_max_open`

An optional maximum number of open connections to the database. If conn_max_idle is greater than 0 and the new conn_max_open is less than conn_max_idle, then conn_max_idle will be reduced to match the new conn_max_open limit. If value <= 0, then there is no limit on the number of open connections. The default is 0 (unlimited).


Type: `int`  


